generate_dag<-function(n_v,n_e,type){
    if (type=='random_forward'){
        U <- matrix(1,n_v,n_v)
        diag(U)<-0
        idx<-which(U==1)
        adjM<-matrix(0,n_v,n_v)
        e<-sample(idx,n_e,replace=FALSE)
        adjM[e]<-1
        adjM<-adjM+t(adjM)
        adjM[lower.tri(adjM)]<-0
    }
    return(adjM)
}

generate_coefM<-function(adjM,emin,emax){
    # generate the beta matrix. first row is intercept, coefM[i+1,j] is beta_i for generating variable j
    coefM<-rbind(colSums(adjM)>0,adjM)
    idx<-coefM==1
    coefM[idx]<-runif(sum(idx),min=emin,max=emax)*
        (rbinom(sum(idx),1,0.6)-0.5)*2
    return(coefM)
}
generate_mvG<-function(coefM,n_o,vari,mode){
    adjM<-(coefM[2:nrow(coefM),]!=0)+0
    D<-matrix(0,n_o,ncol(adjM))
    noisev<-rep(0,1,length(vari))
    for (i in 1:ncol(adjM)){

        if (sum(adjM[,i])==0){
            noisev[i]<-vari[i]
            D[,i]<-rnorm(n_o,0,noisev[i])
        } else {
            if (mode=='marginal'){
            covM<-dgf_to_cov(coefM,noisev)
            noisev[i]<-vari[i]-covM[i,i]
            } else {
                noisev[i]<-vari[i]
            }
            D[,i]<-cbind(matrix(1,nrow=n_o,1),
                         D)%*%coefM[,i]+rnorm(n_o,0,sqrt(noisev[i]))
        }
    }
    
    covM<-dgf_to_cov(coefM,noisev)
    n_v<-ncol(adjM)
    pcM<-matrix(0,n_v,n_v)
    idx<-which(adjM==1)
    for (i in idx){
        ridx <- ((i-1) %% n_v) + 1
        cidx <- floor((i-1) / n_v) + 1
        cond<- setdiff(which(adjM[,cidx]>0),ridx)
        pcM[ridx,cidx]<-covM_to_pcor(covM,ridx,cidx,cond)
    }
    prsqM<-pcM^2
    fsqM<-prsqM/(1-prsqM)
    sk<-((adjM+t(adjM))>0)+0
    out<-list(D=as.data.frame(D),
              pcM=pcM,fsqM=fsqM,
              coefM=coefM,
              adjM=adjM,
              sk=sk,
              noisev=noisev)
    return(out)
}


generate_modified<-function(coefM,n_me,emin,emax,mode){
# generate a modified graph and data given
# inputs: coefM, data generation function
#         number of modified edges (for now number of added edges, if mode=graph). emin and emax are min and max of beta.
	n_v<-dim(coefM)[2]
    if (mode=='graph'){
        coefMm<-coefM[2:(n_v+1),]
        n_v<-ncol(coefM)
        U <- matrix(1,n_v,n_v)
        U[lower.tri(U,diag=T)]<-0
        idx<-intersect(which(U==1),which(coefMm==0))
        aidx<-sample(idx,ceiling(n_me/2),replace=FALSE,prob=1/idx)
        # the prob makes selection later nodes less likely so the graph is less "hubbed" at the end
        didx<-sample(which(coefMm!=0),ceiling(n_me/2),replace=FALSE)
        coefMm[aidx]<-runif(ceiling(n_me/2),min=emin,max=emax)*
            (rbinom(ceiling(n_me/2),1,0.5)-0.5)*2
        coefMm[didx]<-0
        
    }
    return(rbind(coefM[1,],coefMm))
}

run_cds<-function(df,algoId,scoreId,dataType,alpha,penaltyDiscount,mode,nr,ns){
library('rcausal')
# all data
    obj<-tetradrunner(df=df,
             algoId=algoId,
             scoreId=scoreId,
             dataType=dataType,
             alpha=alpha,
             penaltyDiscount=penaltyDiscount,
             verbose=FALSE)
    adjM<-tetrad_to_adjM(obj)
    sk<-((adjM+t(adjM))>0)+0
# bs

    if (mode=='bootstrap'){
        bs_adjM<-matrix(0,dim(df)[2],dim(df)[2])
        bs_sk<-matrix(0,dim(df)[2],dim(df)[2])
        for (i in 1:nr){
            idx<-sample(1:dim(df)[1],dim(df)[1],replace=TRUE)
            obj<-tetradrunner(df=df[idx,],
             algoId=algoId,
             scoreId=scoreId,
             dataType=dataType,
             alpha=alpha,
             penaltyDiscount=penaltyDiscount,
             verbose=FALSE)
            c_adjM<-tetrad_to_adjM(obj)
            bs_adjM<-bs_adjM+c_adjM
            bs_sk<-bs_sk+((c_adjM+t(c_adjM))>0+0)
        }
        bs_adjM<-bs_adjM/nr
        bs_sk<-bs_sk/nr
    } else {
        bs_adjM=c()
        bs_sk=c()
    }
# resample
    if (mode=='resample'){
        rs_adjM<-matrix(0,dim(df)[2],dim(df)[2])
        rs_sk<-matrix(0,dim(df)[2],dim(df)[2])
        for (i in 1:nr){
            idx<-sample(1:dim(df)[1],ns,replace=FALSE)
            obj<-tetradrunner(df=df[idx,],
             algoId=algoId,
             scoreId=scoreId,
             dataType=dataType,
             alpha=alpha,penaltyDiscount=penaltyDiscount,
             verbose=FALSE)
            c_adjM<-tetrad_to_adjM(obj)
            rs_adjM<-rs_adjM+c_adjM
            rs_sk<-rs_sk+((c_adjM+t(c_adjM))>0+0)
        }
        rs_adjM<-rs_adjM/nr
        rs_sk<-rs_sk/nr
    } else {
        rs_adjM=c()
        rs_sk=c()
    }
    
    out<-list(adjM=adjM,
              sk=sk,
              bs_adjM=bs_adjM,
              bs_sk=bs_sk,
              rs_adjM=rs_adjM,
              rs_sk=rs_sk)

    return(out)

}


tetrad_to_el<-function(tobj){
    n_e<-length(tobj$edges)
    if (n_e==0){
        el<-data.frame(node1=character(),
                       edge=character(),
                       node2=character())
    }else{
        el<-as.data.frame(matrix(unlist(strsplit(tobj$edges,' ')),
           n_e,3,
           byrow=TRUE))
        names(el)<-c('node1','edge','node2')
    }
    return(el)
}

tetrad_to_adjM<-function(tobj){
    n_v<-length(tobj$nodes)
    adjM<-matrix(0,n_v,n_v)
    el<-tetrad_to_el(tobj)
    if (dim(el)[1]==0){
    }else{
        for (i in 1:dim(el)[1]){
            if (el$edge[i]=='-->'){
                ridx<-which(tobj$nodes %in% el$node1[i])
                cidx<-which(tobj$nodes %in% el$node2[i])
                adjM[ridx,cidx]=1
            }
            if (el$edge[i]=='---'){
                ridx<-which(tobj$nodes %in% el$node1[i])
                cidx<-which(tobj$nodes %in% el$node2[i])
                adjM[ridx,cidx]=1
                adjM[cidx,ridx]=1
            }
        }
    }
    return(adjM)
}


dgf_to_cov<-function(coefM,noisev){
# coefM is the beta matrix, row 1 is intercept, element i+1,j is the coef of i's comtribution to j. coefM is topographically sorted
# noisev vector of noise variance.
    coef<-coefM[2:nrow(coefM),]
    covD<-matrix(0,ncol(coefM),ncol(coefM))
    for (i in 1:ncol(coefM)){
        if (sum(coef[,i])==0){
            covD[i,i]<-noisev[i]
            covD[i,1:i-1]<-0
        }else{
            for (j in 1:i){
                covD[j,i]<-covD[j,1:i]%*%coef[1:i,i]
                covD[i,j]<-covD[j,i]
                if (i==j){
                    covD[j,i]<-covD[j,i]+noisev[j]
                        }
            }
        }
    }
    return(covD)
}


covM_to_pcor<-function(covM,i,j,condS){
    # input: covM covariance matrix
    # output: partial correlation
    # compute the partial correlation between i, j, given set condS
    M<-covM[c(i,j,condS),c(i,j,condS)]
    pre<-solve(M)
    pcor<--pre[1,2]/sqrt(pre[1,1]*pre[2,2])
    return(pcor)
}

gsobs_compare<-function(gs,obs){

library('PRROC')
# given two binary vectors compute sensitivity specificity ppv npv
# if obs is not a binary vector, threshold it first by aupr.
    if (!(all(obs==1)|all(obs==0))){
        aupro<-pr.curve(scores.class0=obs[gs==1],
                       scores.class1=obs[gs==0],
                       curve=TRUE)
        aupr<-aupro$auc.integral
        aucroc<-roc.curve(scores.class0=obs[gs==1],
                          scores.class1=obs[gs==0])$auc
        idx<-which.max(apply(aupro$curve[,c(1,2)],1,
                             function(x){2*x[1]*x[2]/(x[1]+x[2])}))
        obs<-as.numeric(obs>=aupro$curve[idx,3])
        
        #aupr<-pr.curve(scores.class0=obs[gs==1],
        #              scores.class1=obs[gs==0])$auc.integral
        #aucroc<-roc.curve(scores.class0=obs[gs==1],
        #                  scores.class1=obs[gs==0])$auc
        
        #thre<-sort(obs,decreasing=TRUE)[tn]
        #obs<-as.numeric(obs>=thre)


    }else{
        aupr<-NA
        aucroc<-NA
    }
    tp<-sum((gs==1)&(obs==1))
    fp<-sum((gs==0)&(obs==1))
    tn<-sum((gs==0)&(obs==0))
    fn<-sum((gs==1)&(obs==0))
    sen<-tp/(tp+fn)
    spe<-tn/(tn+fp)
    ppv<-tp/(tp+fp)
    npv<-tn/(tn+fn)
    f1<-2*sen*ppv/(sen+ppv)
    acc<-(tp+tn)/(tp+tn+fp+fn)
    calc_xent = function(gs, obs, eps = 1e-15) {
         predicted = pmin(pmax(obs, eps), 1-eps)
         - (sum(gs * log(predicted) 
                 + (1 - gs) * log(1 - predicted))) / length(gs)
    }
    xentropy<-calc_xent(gs,obs)
    res<-list(sen=sen,spe=spe,ppv=ppv,npv=npv,
              acc=acc,f1=f1,
              aucroc=aucroc,aupr=aupr,xentropy=xentropy)
    
    return(res)
}


gsobs_powersample<-function(gs,obs,gs1_es,gs2_es){
#gs_es is the vector represent the effect size 
#obs is binary vector/continuous vector represent the discovered edges
    if (!all(obs %in% c(0,1))){
        #np<-sum(gs)
        #thre<-sort(obs,decreasing=TRUE)[np]
        #obs<-as.numeric(obs>=thre)
        aupr<-pr.curve(scores.class0=obs[gs==1],
                       scores.class1=obs[gs==0],
                       curve=TRUE)

        idx<-which.max(apply(aupr$curve[,c(1,2)],1,
                             function(x){2*x[1]*x[2]/(x[1]+x[2])}))

        obs<-as.numeric(obs>=aupr$curve[idx,3])
    }

    tpf<-abs(gs1_es-gs2_es)[(gs==1)&(obs==1)]
    res<-cbind(matrix('TP',length(tpf),1),tpf)
    fpf<-gs1_es[(gs==0)&(obs==1)]
    res<-rbind(res,
                cbind(matrix('FP',length(fpf),1),fpf))
    fnf<-abs(gs1_es-gs2_es)[(gs==1)&(obs==0)]
    res<-rbind(res,
                  cbind(matrix('FN',length(fnf),1),fnf))
    return(res)
}
