#!/usr/bin/env Rscript

# function runs simulated experiments given specified graph characteristics
args = commandArgs(trailingOnly=TRUE)
source('graph_tools.R')
source('generate_paired_graphdata.R')


n_v<-as.numeric(args[1])
n_e<-as.numeric(args[2])
n_me<-as.numeric(args[3])
s_rep<-as.numeric(args[4])
e_rep<-s_rep+4
algoId<-args[5]
n_o<-5000

if (algoId=='fges'){
scoreId<-'sem-bic'
dataType<-'continuous'
alpha<-0.05
penaltyDiscount<-2
} else if (algoId=='pc-all'){
algoId<-'pc-all'
scoreId<-'fisher-z-test'
dataType<-'continuous'
alpha<-0.05
penaltyDiscount<-2
}

nrep<-50
n_o1<-c(500,1000,2000,5000)
r_o2<-c(0.1,0.2,0.5,1)

emin<-0.1
emax<-0.35
margv<-rep(1,1,n_v)

pname<-paste0('../Results/',algoId,'/','nv',n_v,'ne',n_e,'nme',n_me)

dir.create(pname,recursive=TRUE)
for (i in s_rep:e_rep){
    if (!file.exists(paste0(pname,'/gs_e',i))){
        out<-generate_paired_graphdata(n_v,n_e,n_me,n_o,
                                       emin,emax,margv)
        gen1<-out$gen1
        gen2<-out$gen2
        save(file=paste0(pname,'/gs_e',i),gen1,gen2)
    } else {
        load(file=paste0(pname,'/gs_e',i))
    }

    for (n1 in n_o1){
        if (!file.exists(paste0(pname,'/res_e',i,'_g1_n',n1,'_bs'))){
            out1<-run_cds(df=gen1$D[1:n1,],
             algoId=algoId,
             scoreId=scoreId,
             dataType=dataType,
             alpha=alpha,
             penaltyDiscount=penaltyDiscount,
             mode='bootstrap',
             nr=50)
            save(file=paste0(pname,'/res_e',i,'_g1_n',n1,'_bs'),out1)
        }
        
        for (n2 in r_o2*n1){
            if (!file.exists(paste0(pname,'/res_e',i,'_g2_n',n2,'_bs'))){
                out2<-run_cds(df=gen2$D[1:n2,],
                              algoId=algoId,
                              scoreId=scoreId,
                              dataType=dataType,
                              alpha=alpha,
                              penaltyDiscount=penaltyDiscount,
                              mode='bootstrap',
                              nr=50)
                save(file=paste0(pname,'/res_e',i,'_g2_n',n2,'_bs'),out2)
            }
            if (!file.exists(paste0(pname,'/res_e',i,'_g1_n',n1,'_rs',n2))){
                if (n1!=n2){
					out1<-run_cds(df=gen1$D[1:n1,],
                              algoId=algoId,
                              scoreId=scoreId,
                              dataType=dataType,
                              alpha=alpha,
                              penaltyDiscount=penaltyDiscount,
                              mode='resample',
                              nr=50,ns=n2)
                save(file=paste0(pname,'/res_e',i,'_g1_n',n1,'_rs',n2),
                     out1)
		}
            }
        }
    }

}

